﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                              GPLUS class                                 |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 11-23-2006 (MM-DD-YYYY)            |
//|                        Last revised : 11-23-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.IO;

using System.Drawing.Imaging;

namespace Gplus
{
    class GP
    {
        private const string GDIPLUSEXT = ".DIB.EMF.GIF.JPG.JPEG.PNG.TIF.TIFF.WMF.BMP.ICO.";
        private const string Dot = ".";

        public static void GetImageSizeFromFile(string ImageName, ref int imgW, ref int imgH)
        {
            imgW = 0; imgH = 0;
            if (File.Exists(ImageName))
            {
                int So = ImageName.LastIndexOf(Dot);
                if (So > -1)
                {// Check for valid extension
                    string ImageType = ImageName.Substring(So, ImageName.Length - So).ToUpper() + Dot;
                    if (GDIPLUSEXT.IndexOf(ImageType) > -1)
                    {// Load image to get its size
                        Bitmap Img = new Bitmap(ImageName);
                        imgW = Img.Width; imgH = Img.Height;
                        Img.Dispose(); // free up image
                    }
                }
            }
        }

        public unsafe static Bitmap Alphablend(Bitmap Img, Bitmap Msk)
        {
            int imgW = Img.Width, imgH = Img.Height;

            // Create empty bitmap to store the result
            Bitmap Mix = new Bitmap(imgW, imgH);

            Rectangle bounds = new Rectangle(0, 0, imgW, imgH);
            // Lock BitmapData
            BitmapData bdMix = Mix.LockBits(bounds, ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            uint* ppMix = (uint*)bdMix.Scan0.ToPointer();
            BitmapData bdImg = Img.LockBits(bounds, ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            uint* ppImg = (uint*)bdImg.Scan0.ToPointer();
            BitmapData bdMsk = Msk.LockBits(bounds, ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            uint* ppMsk = (uint*)bdMsk.Scan0.ToPointer();

            byte* pMix = (byte*)ppMix;
            byte* pImg = (byte*)ppImg;
            byte* pMsk = (byte*)ppMsk;

            for (int y = 0; y < imgH; y++)
            {
                for (int x = 0; x < imgW; x++)
                {
                    // Alpha channel is based on shade of gray ranging from 0-255
                    // in case we use a color mask, make sure to convert it first to shade of gray.
                    byte Alpha = (byte)((int)(pMsk[2] * 0.3 + pMsk[1] * 0.59 + pMsk[0] * 0.11) & 0x000000FF);

                    // GDImage AlphaGain is not being used there
                    //Alpha = MIN&(Alpha * AlphaGain, 255) 

                    float AlphaCoef = ((float)Alpha / 255f);

                    pMix[3] = Alpha;
                    pMix[2] = (byte)(pImg[2] * AlphaCoef);
                    pMix[1] = (byte)(pImg[1] * AlphaCoef);
                    pMix[0] = (byte)(pImg[0] * AlphaCoef);

                    pMix += 4;
                    pImg += 4;
                    pMsk += 4;
                }
            }

            Img.UnlockBits(bdImg); Img.Dispose();
            Msk.UnlockBits(bdMsk); Msk.Dispose();
            Mix.UnlockBits(bdMix);

            return Mix;
        }

    }
}
